/* DWARF2 EH unwinding support for x86.
   Copyright (C) 2007 Free Software Foundation, Inc.

This file is part of GCC.

GCC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

In addition to the permissions in the GNU General Public License, the
Free Software Foundation gives you unlimited permission to link the
compiled version of this file with other programs, and to distribute
those programs without any restriction coming from the use of this
file.  (The General Public License restrictions do apply in other
respects; for example, they cover modification of the file, and
distribution when not linked into another program.)

GCC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING.  If not, write to
the Free Software Foundation, 51 Franklin Street, Fifth Floor,
Boston, MA 02110-1301, USA.  */

#define MD_FALLBACK_FRAME_STATE_FOR sol2_fallback_frame_state

#include <signal.h>
#include <siginfo.h>
#include <sys/ucontext.h>
#include <sys/regset.h>

static _Unwind_Reason_Code
sol2_fallback_frame_state (struct _Unwind_Context *context,
			   _Unwind_FrameState *fs)
{
  static int eh_debug = -1;

  unsigned char *ra = context->ra;
  ucontext_t *uctx;
  mcontext_t *mctx;
  long new_cfa;
  
  int context_is_sighandler;

  if (eh_debug == -1)
    eh_debug = getenv ("EH_DEBUG") != 0;

  if (eh_debug)
    printf ("FALLBACK # ra = 0x%p, cfa = 0x%p...\n",
	    context->ra, context->cfa);

  context_is_sighandler =
    /* Solaris 2.8 - mono thread
       -------------------------
       <sigacthandler+17>:  mov    0x10(%ebp),%esi
       <sigacthandler+20>:  push   %esi
       <sigacthandler+21>:  pushl  0xc(%ebp)
       <sigacthandler+24>:  mov    0x8(%ebp),%ecx
       <sigacthandler+27>:  push   %ecx
       <sigacthandler+28>:  mov    0x244(%ebx),%eax
       <sigacthandler+34>:  call   *(%eax,%ecx,4)
       <sigacthandler+37>:  add    $0xc,%esp        <--- context->ra
       <sigacthandler+40>:  push   %esi ... */
    (*(unsigned long *)(ra - 20) == 0x5610758b
     && *(unsigned long *)(ra - 16) == 0x8b0c75ff
     && *(unsigned long *)(ra - 12) == 0x8b51084d
     && *(unsigned long *)(ra - 8)  == 0x00024483
     && *(unsigned long *)(ra - 4)  == 0x8814ff00
     && *(unsigned long *)(ra - 0)  == 0x560cc483)

    ||  /* Solaris 2.8 - multi threads
	   ---------------------------
	   <__sighndlr+0>:      push   %ebp
	   <__sighndlr+1>:      mov    %esp,%ebp
	   <__sighndlr+3>:      pushl  0x10(%ebp)
	   <__sighndlr+6>:      pushl  0xc(%ebp)
	   <__sighndlr+9>:      pushl  0x8(%ebp)
	   <__sighndlr+12>:     call   *0x14(%ebp)
	   <__sighndlr+15>:     leave               <--- context->ra  */
    (*(unsigned long *)(ra - 15) == 0xffec8b55
     && *(unsigned long *)(ra - 11) == 0x75ff1075
     && *(unsigned long *)(ra - 7)  == 0x0875ff0c
     && *(unsigned long *)(ra - 3)  == 0xc91455ff)

    ||  /* Solaris 2.10
	   ------------
	   <__sighndlr+0>:      push   %ebp
	   <__sighndlr+1>:      mov    %esp,%ebp
	   <__sighndlr+3>:      pushl  0x10(%ebp)
	   <__sighndlr+6>:      pushl  0xc(%ebp)
	   <__sighndlr+9>:      pushl  0x8(%ebp)
	   <__sighndlr+12>:     call   *0x14(%ebp)
	   <__sighndlr+15>:     add    $0xc,%esp     <--- context->ra
	   <__sighndlr+18>:     leave
	   <__sighndlr+19>:     ret  */
    (*(unsigned long *)(ra - 15) == 0xffec8b55
     && *(unsigned long *)(ra - 11) == 0x75ff1075
     && *(unsigned long *)(ra - 7)  == 0x0875ff0c
     && *(unsigned long *)(ra - 3)  == 0x831455ff
     && *(unsigned long *)(ra + 1)  == 0xc3c90cc4);

  if (context_is_sighandler)
    {
      typedef struct {
	int signo;
	siginfo_t  * siginfo;
	ucontext_t * ucontext;
      } handler_args_t;

      handler_args_t * handler_args = context->cfa;

      uctx = handler_args->ucontext;
      mctx = &uctx->uc_mcontext;
    }
  else
    return _URC_END_OF_STACK;

  if (eh_debug)
    printf ("FALLBACK - uctx = 0x%p, mctx = 0x%p\n", uctx, mctx);

  new_cfa = mctx->gregs[UESP];

  fs->cfa_how = CFA_REG_OFFSET;
  fs->cfa_reg = 4;
  fs->cfa_offset = new_cfa - (long) context->cfa;

  /* The SVR4 register numbering macros aren't usable in libgcc.  */
  fs->regs.reg[0].how = REG_SAVED_OFFSET;
  fs->regs.reg[0].loc.offset = (long)&mctx->gregs[EAX] - new_cfa;
  fs->regs.reg[3].how = REG_SAVED_OFFSET;
  fs->regs.reg[3].loc.offset = (long)&mctx->gregs[EBX] - new_cfa;
  fs->regs.reg[1].how = REG_SAVED_OFFSET;
  fs->regs.reg[1].loc.offset = (long)&mctx->gregs[ECX] - new_cfa;
  fs->regs.reg[2].how = REG_SAVED_OFFSET;
  fs->regs.reg[2].loc.offset = (long)&mctx->gregs[EDX] - new_cfa;
  fs->regs.reg[6].how = REG_SAVED_OFFSET;
  fs->regs.reg[6].loc.offset = (long)&mctx->gregs[ESI] - new_cfa;
  fs->regs.reg[7].how = REG_SAVED_OFFSET;
  fs->regs.reg[7].loc.offset = (long)&mctx->gregs[EDI] - new_cfa;
  fs->regs.reg[5].how = REG_SAVED_OFFSET;
  fs->regs.reg[5].loc.offset = (long)&mctx->gregs[EBP] - new_cfa;
  fs->regs.reg[8].how = REG_SAVED_OFFSET;
  fs->regs.reg[8].loc.offset = (long)&mctx->gregs[EIP] - new_cfa;
  fs->retaddr_column = 8;

  if (eh_debug)
    printf ("FALLBACK - ncfa = 0x%p, nip = 0x%p\n",
	    (void *)new_cfa, (void *)mctx->gregs[EIP]);

  return _URC_NO_REASON;
}

