/* DWARF2 EH unwinding support for PowerPC VxWorks.
   Copyright (C) 2006 Free Software Foundation, Inc.

   This file is part of GCC.

   GCC is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 2, or (at your
   option) any later version.

   In addition to the permissions in the GNU General Public License,
   the Free Software Foundation gives you unlimited permission to link
   the compiled version of this file with other programs, and to
   distribute those programs without any restriction coming from the
   use of this file.  (The General Public License restrictions do
   apply in other respects; for example, they cover modification of
   the file, and distribution when not linked into another program.)

   GCC is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with GCC; see the file COPYING.  If not, write to the
   Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
   MA 02110-1301, USA.  */

/* Do code reading to identify a signal frame, and set the frame
   state data appropriately.  See unwind-dw2.c for the structs.  */

#define MD_FALLBACK_FRAME_STATE_FOR ppc_vxworks_fallback_frame_state

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <signal.h>

/* We need the constant and structure definitions describing the machine
   state.  Part of this is normally retrieved from the VxWorks "regs.h" but
   #including it here gets the GCC internals instance of this file instead.
   We need to #include the version we need directly here, and prevent the
   possibly indirect inclusion of the GCC one, as its contents is useless to
   us and it depends on several other headers that we don't have at hand.  */
#include <arch/ppc/regsPpc.h>
#define GCC_REGS_H

#include <sigLib.h>

typedef struct sigcontext sigcontext_t;

/* Determine whether CONTEXT designates a signal handler, and return the
   associated sigontext address if so.  Return NULL otherwise.  */

static sigcontext_t *
sigcontext_for (struct _Unwind_Context *context)
{
  const unsigned int * ra = context->ra;

  /* VxWorks 5.5.  */
  if ((*(ra - 6) == 0x7fe3fb78      /* or     r3,r31,r31 */
       && *(ra - 5) == 0x389e000c   /* addi   r4,r30,0xc */
       && *(ra - 4) == 0x7fa803a6   /* mtspr  LR,r29 */
       && *(ra - 3) == 0x7fc5f378   /* or     r5,r30,r30 */
       && *(ra - 2) == 0x4cc63182   /* crxor  crb6,crb6,crb6 */
       && *(ra - 1) == 0x4e800021)  /* blrl */
      ||
      (*(ra - 6) == 0x7fe3fb78      /* or     r3,r31,r31 */
       && *(ra - 5) == 0x809e0014   /* lwz    r4,20(r30) */
       && *(ra - 4) == 0x7fc5f378   /* or     r5,r30,r30 */
       && *(ra - 3) == 0x7fa803a6   /* mtspr  LR,r29 */
       && *(ra - 2) == 0x4cc63182   /* crxor  crb6,crb6,crb6 */
       && *(ra - 1) == 0x4e800021   /* blrl */
       && *(ra - 0) == 0x7fc3f378)) /* or     r3,r30,r30 */
    {
	struct {
	    char pad [0x40];
	    sigcontext_t sigcontext;
	} * frame = context->cfa;
	return &frame->sigcontext;
    }

  return 0;
}

/* The fallback proper.  */

#ifdef DWARF_ALT_FRAME_RETURN_COLUMN
#define RETURN_COLUMN DWARF_ALT_FRAME_RETURN_COLUMN
#else
#define RETURN_COLUMN ARG_POINTER_REGNUM
#endif

#define LR_REGNO LINK_REGISTER_REGNUM

#define REGISTER_CFA_OFFSET_FOR(FS,REGNO,ADDR,CFA)\
do { \
(FS)->regs.reg[REGNO].how = REG_SAVED_OFFSET; \
(FS)->regs.reg[REGNO].loc.offset = (long) (ADDR) - (CFA); \
} while (0);

static _Unwind_Reason_Code
ppc_vxworks_fallback_frame_state (struct _Unwind_Context *context,
				  _Unwind_FrameState *fs)
{
  static int eh_debug = -1;

  sigcontext_t * sigctx = sigcontext_for (context);
  REG_SET * mctx;

  long new_cfa;
  int i;

  if (eh_debug == -1)
    {
      char *eh_debug_env = getenv ("EH_DEBUG");
      eh_debug = eh_debug_env ? atoi (eh_debug_env) : 0;
   }
  
  if (eh_debug)
    printf ("FALLBACK called for CFA = %p, RA = %p, SIGCTX = %p\n",
	    context->cfa, context->ra, sigctx);

  if (sigctx == NULL)
    return _URC_END_OF_STACK;

  mctx = sigctx->sc_pregs;

  /* The "kernel" frame cfa is the stack pointer at the signal occurrence
     point.  */
  new_cfa = mctx->gpr[STACK_POINTER_REGNUM];

  fs->cfa_how = CFA_REG_OFFSET;
  fs->cfa_reg = STACK_POINTER_REGNUM;
  fs->cfa_offset = new_cfa - (long) context->cfa;

  /* And we state how to find the various registers it has saved with
     relative offset rules from there.  */

  for (i = 0; i < 32; i++)
    if (i != STACK_POINTER_REGNUM)
      REGISTER_CFA_OFFSET_FOR (fs, i, &mctx->gpr[i], new_cfa);

  REGISTER_CFA_OFFSET_FOR (fs, CR2_REGNO, &mctx->cr, new_cfa);
  REGISTER_CFA_OFFSET_FOR (fs, XER_REGNO, &mctx->xer, new_cfa);
  REGISTER_CFA_OFFSET_FOR (fs, LR_REGNO, &mctx->lr, new_cfa);

  fs->retaddr_column = RETURN_COLUMN;
  REGISTER_CFA_OFFSET_FOR (fs, RETURN_COLUMN, &mctx->pc, new_cfa);

  return _URC_NO_REASON;
}

